@@ -188,7 +188,8 @@ class AbstractEtcdClientWithFailover(etcd.Client):
                 logger.debug("Retrieved list of machines: %s", machines)
                 if machines:
                     random.shuffle(machines)
-                    self._update_dns_cache(self._dns_resolver.resolve_async, machines)
+                    if not self._use_proxies:
+                        self._update_dns_cache(self._dns_resolver.resolve_async, machines)
                     return machines
             except Exception as e:
                 self.http.clear()
@@ -282,13 +283,14 @@ class AbstractEtcdClientWithFailover(etcd.Client):
         except DNSException:
             return []
 
-    def _get_machines_cache_from_srv(self, srv):
+    def _get_machines_cache_from_srv(self, srv, srv_suffix=None):
         """Fetch list of etcd-cluster member by resolving _etcd-server._tcp. SRV record.
         This record should contain list of host and peer ports which could be used to run
         'GET http://{host}:{port}/members' request (peer protocol)"""
 
         ret = []
         for r in ['-client-ssl', '-client', '-ssl', '', '-server-ssl', '-server']:
+            r = '{0}-{1}'.format(r, srv_suffix) if srv_suffix else r
             protocol = 'https' if '-ssl' in r else 'http'
             endpoint = '/members' if '-server' in r else ''
             for host, port in self.get_srv_record('_etcd{0}._tcp.{1}'.format(r, srv)):
@@ -325,7 +327,7 @@ class AbstractEtcdClientWithFailover(etcd.Client):
 
         machines_cache = []
         if 'srv' in self._config:
-            machines_cache = self._get_machines_cache_from_srv(self._config['srv'])
+            machines_cache = self._get_machines_cache_from_srv(self._config['srv'], self._config.get('srv_suffix'))
 
         if not machines_cache and 'hosts' in self._config:
             machines_cache = list(self._config['hosts'])
@@ -404,7 +406,7 @@ class EtcdClient(AbstractEtcdClientWithFailover):
         if self.http is not None:
             try:
                 self.http.clear()
-            except (ReferenceError, TypeError):
+            except (ReferenceError, TypeError, AttributeError):
                 pass
 
     def _prepare_get_members(self, etcd_nodes):
@@ -602,9 +604,24 @@ class Etcd(AbstractEtcd):
             history = nodes.get(self._HISTORY)
             history = history and TimelineHistory.from_node(history.modifiedIndex, history.value)
 
-            # get last leader operation
-            last_leader_operation = nodes.get(self._LEADER_OPTIME)
-            last_leader_operation = 0 if last_leader_operation is None else int(last_leader_operation.value)
+            # get last know leader lsn and slots
+            status = nodes.get(self._STATUS)
+            if status:
+                try:
+                    status = json.loads(status.value)
+                    last_lsn = status.get(self._OPTIME)
+                    slots = status.get('slots')
+                except Exception:
+                    slots = last_lsn = None
+            else:
+                last_lsn = nodes.get(self._LEADER_OPTIME)
+                last_lsn = last_lsn and last_lsn.value
+                slots = None
+
+            try:
+                last_lsn = int(last_lsn)
+            except Exception:
+                last_lsn = 0
 
             # get list of members
             members = [self.member(n) for k, n in nodes.items() if k.startswith(self._MEMBERS) and k.count('/') == 1]
@@ -626,9 +643,9 @@ class Etcd(AbstractEtcd):
             sync = nodes.get(self._SYNC)
             sync = SyncState.from_node(sync and sync.modifiedIndex, sync and sync.value)
 
-            cluster = Cluster(initialize, config, leader, last_leader_operation, members, failover, sync, history)
+            cluster = Cluster(initialize, config, leader, last_lsn, members, failover, sync, history, slots)
         except etcd.EtcdKeyNotFound:
-            cluster = Cluster(None, None, None, None, [], None, None, None)
+            cluster = Cluster(None, None, None, None, [], None, None, None, None)
         except Exception as e:
             self._handle_exception(e, 'get_cluster', raise_ex=EtcdError('Etcd is not responding properly'))
         self._has_failed = False
@@ -665,8 +682,12 @@ class Etcd(AbstractEtcd):
         return self._client.write(self.config_path, value, prevIndex=index or 0)
 
     @catch_etcd_errors
-    def _write_leader_optime(self, last_operation):
-        return self._client.set(self.leader_optime_path, last_operation)
+    def _write_leader_optime(self, last_lsn):
+        return self._client.set(self.leader_optime_path, last_lsn)
+
+    @catch_etcd_errors
+    def _write_status(self, value):
+        return self._client.set(self.status_path, value)
 
     @catch_etcd_errors
     def _update_leader(self):
